package ru.myitschool.work.ui.admin.search

import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewModelScope
import androidx.lifecycle.viewmodel.CreationExtras
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import androidx.paging.cachedIn
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import ru.myitschool.work.data.AdminRepositoryImpl
import ru.myitschool.work.data.PassRepositoryImpl
import ru.myitschool.work.data.UserRepositoryImpl
import ru.myitschool.work.data.local.CredentialsLocalDataSource
import ru.myitschool.work.data.local.UserLocalDataSource
import ru.myitschool.work.data.network.AdminNetworkDataSource
import ru.myitschool.work.data.network.PassNetworkDataSource
import ru.myitschool.work.data.network.UserNetworkDataSource
import ru.myitschool.work.domain.admin.BlockUserUseCase
import ru.myitschool.work.domain.admin.UnblockUserUseCase
import ru.myitschool.work.domain.entities.PassEntity
import ru.myitschool.work.domain.entities.UserEntity
import ru.myitschool.work.domain.passes.GetUsersPassesUseCase
import ru.myitschool.work.domain.user.GetUserByLoginUseCase
import ru.myitschool.work.ui.admin.view.UsersPassesPagingSource

class AdminViewModel(
    private val getUserByLoginUseCase: GetUserByLoginUseCase,
    private val getUsersPassesUseCase: GetUsersPassesUseCase,
    private val unBlockUserUseCase: UnblockUserUseCase,
    private val blockUserUseCase: BlockUserUseCase
) : ViewModel() {

    private val _state = MutableStateFlow<State>(State.Waiting)
    val state = _state.asStateFlow()

//    private var _listState: Flow<PagingData<PassEntity>>? = null
//    val listState: Flow<PagingData<PassEntity>> get() = _listState!!

    private var currentLogin: String = "pivanov"

    fun onFind(login: String) {
        viewModelScope.launch {
            _state.emit(State.Loading)
            getUserByLoginUseCase(login).fold(
                onSuccess = { data ->
                    currentLogin = login
                    _state.emit(State.Show(data))
                    setUpPager(login)
                },
                onFailure = { _state.emit(State.Error(it.message.toString())) }
            )
        }
    }

    fun onRefresh() {
        updateState()
    }

    private fun updateState() {
        viewModelScope.launch {
            _state.emit(State.Loading)
            getUserByLoginUseCase(currentLogin!!).fold(
                onSuccess = { _state.emit(State.Show(it)) },
                onFailure = { _state.emit(State.Error(it.message.toString())) }
            )
        }
    }

    private fun setUpPager(login: String) {
//        _listState = Pager(
//            config = PagingConfig(
//                pageSize = 10,
//                enablePlaceholders = false,
//                maxSize = 50
//            )
//        ) {
//            UsersPassesPagingSource(getUsersPassesUseCase::invoke, login)
//        }.flow
//            .cachedIn(viewModelScope)
    }

    fun onBlock() {
        viewModelScope.launch {
            blockUserUseCase(currentLogin!!)
        }
    }

    fun unblock() {
        viewModelScope.launch {
            unBlockUserUseCase(currentLogin!!)
        }
    }

    sealed interface State {
        data class Show(val user: UserEntity) : State
        data object Waiting : State
        data object Loading : State
        data class Error(val errorMessage: String) : State
    }


    companion object {
        val Factory: ViewModelProvider.Factory = object : ViewModelProvider.Factory {
            @Suppress("UNCHECKED_CAST")
            override fun <T : ViewModel> create(modelClass: Class<T>, extras: CreationExtras): T {
                val adminRepository = AdminRepositoryImpl(
                    networkDataSource = AdminNetworkDataSource,
                    localCredentialsLocalDataSource = CredentialsLocalDataSource.getInstance()
                )
                return AdminViewModel(
                    getUserByLoginUseCase = GetUserByLoginUseCase(
                        repository = UserRepositoryImpl(
                            credentialsLocalDataSource = CredentialsLocalDataSource.getInstance(),
                            userLocalDataSource = UserLocalDataSource,
                            networkDataSource = UserNetworkDataSource
                        )
                    ),
                    getUsersPassesUseCase = GetUsersPassesUseCase(
                        repository = PassRepositoryImpl(
                            networkDataSource = PassNetworkDataSource,
                            credentialsLocalDataSource = CredentialsLocalDataSource.getInstance()
                        )
                    ),
                    blockUserUseCase = BlockUserUseCase(
                        repository = adminRepository
                    ),
                    unBlockUserUseCase = UnblockUserUseCase(
                        repository = adminRepository
                    )
                ) as T
            }
        }
    }
}