import android.content.Context
import androidx.test.core.app.ApplicationProvider
import kotlinx.coroutines.test.runTest
import org.junit.Assert.assertEquals
import org.junit.Assert.assertNotNull
import org.junit.Assert.assertNull
import org.junit.Before
import org.junit.Test
import org.junit.runner.RunWith
import org.robolectric.RobolectricTestRunner
import ru.myitschool.work.core.Constants
import ru.myitschool.work.data.UserRepositoryImpl
import ru.myitschool.work.data.local.CredentialsLocalDataSource
import ru.myitschool.work.data.local.UserLocalDataSource
import ru.myitschool.work.data.network.UserNetworkDataSource
import ru.myitschool.work.domain.entities.UserEntity

@RunWith(RobolectricTestRunner::class)
class UserRepositoryImplTest {

    private lateinit var context: Context
    private lateinit var repository: UserRepositoryImpl

    @Before
    fun setUp() {
        context = ApplicationProvider.getApplicationContext()
        CredentialsLocalDataSource.buildSource(
            context.getSharedPreferences(
                Constants.TOKEN_KEY,
                Context.MODE_PRIVATE
            )
        )
        repository = UserRepositoryImpl(
            credentialsLocalDataSource = CredentialsLocalDataSource.getInstance(),
            userLocalDataSource = UserLocalDataSource,
            networkDataSource = UserNetworkDataSource
        )
    }

    @Test
    fun `When user exist return true`() = runTest {
        assertEquals(true, repository.isUserExist("pivanov").getOrNull())
    }

    @Test
    fun `When user does not exist return false`() = runTest {
        assertEquals(false, repository.isUserExist("pivanov").getOrNull())
    }

    @Test
    fun `When user existing and password matches success login`() = runTest {
        assertEquals(
            UserEntity(
                isAdmin = true,
                name = "",
                lastVisit = "",
                photoUrl = "",
                position = "",
                isCardBlocked = false
            ), repository.login("pivanov", "admin").getOrNull()
        )
    }

    @Test
    fun `When user enter incorrect login`() = runTest {
        assertNull(repository.login("***", "admin").getOrNull())
    }

    @Test
    fun `When user enter incorrect password`() = runTest {
        assertNull(repository.login("pivanov", "***").getOrNull())
    }

    @Test
    fun `When user enter incorrect login and password`() = runTest {
        assertNull(repository.login("***", "***").getOrNull())
    }

    @Test
    fun `When user logouts local credentials wipes`() = runTest {
        repository.logout()
        runCatching {
            CredentialsLocalDataSource.getInstance().getToken()
        }.let { assert(it.isFailure) }
    }

    @Test
    fun `Get user by login returns correct user`() = runTest {
        CredentialsLocalDataSource.getInstance().updateToken("pivanov", "admin")
        assertEquals(
            UserEntity(
                isAdmin = false,
                name = "",
                lastVisit = "",
                photoUrl = "",
                position = "",
                isCardBlocked = false
            ),
            repository.getUserByLogin("").getOrNull()
        )
    }

    @Test
    fun `Get user by login works with admin roots`() = runTest {
        CredentialsLocalDataSource.getInstance().updateToken("pivanov", "admin")
        assertNotNull(repository.getUserByLogin("").getOrNull())
    }

    @Test
    fun `Get user by login don not works without admin roots`() = runTest {
        CredentialsLocalDataSource.getInstance().updateToken("", "")
        assertNull(repository.getUserByLogin("").getOrNull())
    }

    @Test
    fun `When user successfully logged for the first time it will authorize automatically`() =
        runTest {
            val loggedUser = repository.login("pivanov", "admin").getOrNull()
            assertEquals(
                loggedUser,
                repository.authorize(CredentialsLocalDataSource.getInstance().getToken())
            )
        }

    @Test
    fun `When user successfully logged its info saved locally for current session`() = runTest {
        val loggedUser = repository.login("pivanov", "admin").getOrNull()
        assertEquals(loggedUser, repository.getCurrentUser())
    }
}